<?php

namespace common\logic\Payment\Service;

use common\logic\AbstractBuilder;
use common\logic\Order\Order\Model\Order;
use common\logic\Order\Order\Service\OrderSolver;
use common\logic\Payment\Model\Payment;
use common\logic\Producer\Producer\Model\Producer;
use common\logic\User\User\Model\User;

class PaymentBuilder extends AbstractBuilder
{
    protected PaymentSolver $paymentSolver;
    protected OrderSolver $orderSolver;

    public function loadDependencies(): void
    {
        $this->paymentSolver = $this->loadService(PaymentSolver::class);
        $this->orderSolver = $this->loadService(OrderSolver::class);
    }

    public function instanciatePayment(
        string $type,
        float $amount,
        Producer $producer,
        User $user,
        User $userAction,
        string $meanPayment = null,
        Order $order = null
    ): Payment
    {
        $payment = new Payment;

        $payment->type = $type;
        $payment->amount = round($amount, 2);
        $payment->populateProducer($producer);
        $payment->populateUser($user);
        $payment->populateUserAction($userAction);

        if($order) {
            $payment->populateOrder($order);
        }

        if($meanPayment) {
            $payment->mean_payment = $meanPayment;
        }

        return $payment;
    }

    public function createPayment(
        string $type,
        float $amount,
        Producer $producer,
        User $user,
        User $userAction,
        string $meanPayment = null,
        Order $order = null
    ): ?Payment
    {
        if ($amount > -0.01 && $amount < 0.01) {
            return null;
        }

        $payment = $this->instanciatePayment($type, $amount, $producer, $user, $userAction, $meanPayment, $order);
        $payment->setComment($payment->getComment() . $this->orderSolver->getPaymentComment($payment));
        $this->create($payment);

        return $payment;
    }
}